import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Badge, Button, IconButton, ListItemDecorator, Menu, MenuItem, Sheet, Typography, useTheme, Box, Dropdown, MenuButton, CircularProgress, Tabs, TabList, Tab, TabPanel, MenuList, List, tabClasses, Grid, ListItemButton, ListItem, ListItemContent, ListDivider, Tooltip, Input, Card, } from "@mui/joy";
import { ClickAwayListener, Dialog, DialogActions, Popper, useMediaQuery, } from "@mui/material";
import { SearchPreset, } from "@nerdjs/nerd-core";
import { useEffect, useRef, useState, useMemo, Fragment, useCallback, } from "react";
import { Trans, useTranslation } from "react-i18next";
import { useSearchParams } from "react-router-dom";
import { Searchable } from "./searchable";
import { genericIcons } from "./utils";
import nerdUII18n from "../i18n";
import { SearchPresetForm } from "./searchPresetForm";
import { useSelector } from "react-redux";
import { nerdAdvancedSearchSelectedPresetsSelector, setSelectedPresets, } from "../redux/nerdAdvancedSearch";
import { useDispatch } from "react-redux";
const DEFAULT_SEARCH_PRESET = {
    keyword: "sales_loads",
    global: false,
    color: "",
    name: "",
    order: 0,
    filter: "",
};
/**
 *
 * @param {AdvancedSearchProps} props component props
 * @returns {ReactElement} NerdAdvancedSearch
 */
export function NerdAdvancedSearch({ definitions, open = false, setOpen, variant = "toolbar", buttonFaIcon = "fa-solid fa-magnifying-glass", onNewSearchPreset, onDeleteSearchPreset, searchMetadata, onRefresh, onFilterChange, refreshPending, searchPresets, currentUserUUID, hidePresets, itemsPerPage, }) {
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    const dispatch = useDispatch();
    const theme = useTheme();
    const sm = useMediaQuery(theme.breakpoints.down("sm"));
    const [_open, _setOpen] = useState(open);
    const [_variant, _setVariant] = useState(variant ?? localStorage.getItem("searchVariant"));
    const [_advancedMode, _setAdvancedMode] = useState(false);
    const _definitions = _advancedMode && searchMetadata ? searchMetadata : definitions;
    const [searchParams, setSearchParams] = useSearchParams();
    const filtersString = searchParams.get("filters");
    const [filters, setFilters] = useState([]);
    const ref = useRef();
    const filterItemsContainerRef = useRef();
    const [openPresetForm, setOpenPresetForm] = useState(null);
    const inputRef = useRef(null);
    const filtersButtonRef = useRef(null);
    const [searchValue, setSearchValue] = useState("");
    const [openFiltersList, setOpenFiltersList] = useState(false);
    useEffect(() => {
        if (filtersString) {
            try {
                setFilters(JSON.parse(filtersString));
            }
            catch {
                return;
            }
        }
    }, [filtersString]);
    useEffect(() => {
        if (sm) {
            _setVariant("dialog");
        }
        else {
            _setVariant("toolbar");
        }
    }, [sm]);
    useEffect(() => {
        localStorage.setItem("searchVariant", _variant);
        // if (_variant === "dialog") _setOpen(true);
    }, [_variant]);
    useEffect(() => {
        if (setOpen)
            setOpen(_open);
    }, [_open]);
    useEffect(() => {
        if (open != undefined)
            _setOpen(open);
    }, [open]);
    const clearAll = () => {
        handleFiltersChange([]);
        dispatch(setSelectedPresets([]));
    };
    const handleAddField = useCallback((s) => {
        setTimeout(() => {
            handleFiltersChange([
                ...filters,
                {
                    name: s.name,
                    comparison: s.type === "string" ? "like" : "eq",
                    value: s.type === "bool" ? true : "",
                    isNew: true,
                },
            ]);
            setTimeout(() => {
                if (filterItemsContainerRef.current)
                    filterItemsContainerRef.current.scrollTo(filterItemsContainerRef.current.scrollWidth, 0);
            }, 200);
        }, 100);
    }, [filters]);
    const handleFiltersChange = (filters) => {
        setFilters(filters);
        dispatch(setSelectedPresets([]));
        onFilterChange?.(filters.length > 0
            ? JSON.stringify(filters, (key, value) => key === "isNew" ? undefined : value)
            : null);
        setSearchParams((prevState) => {
            prevState.set("offset", "0");
            if (filters.length > 0) {
                prevState.set("filters", JSON.stringify(filters, (key, value) => key === "isNew" ? undefined : value));
            }
            else {
                prevState.delete("filters");
            }
            return prevState;
        });
    };
    return (_jsxs(Box, { ref: ref, width: "100%", children: [_jsxs(Sheet, { sx: {
                    py: 0.5,
                    px: 1,
                    borderRadius: 1,
                    display: "flex",
                    justifyContent: "space-between",
                    alignItems: "center",
                    width: "100%",
                    zIndex: 99,
                    gap: 1,
                }, children: [_variant === "dialog" ? (_jsx(Badge, { badgeContent: filters.length, size: "sm", color: "danger", badgeInset: "8%", children: _jsx(IconButton, { variant: "solid", color: "primary", size: "sm", onClick: () => _setOpen((o) => !o), children: _jsx("i", { style: { fontSize: 14 }, className: buttonFaIcon }) }) })) : null, _variant === "toolbar" ? (_jsxs(_Fragment, { children: [hidePresets ? null : (_jsx(Presets, { currentUserUUID: currentUserUUID, searchPresets: searchPresets, setFilters: handleFiltersChange, setOpenPresetForm: (preset) => setOpenPresetForm(preset) })), filters.length > 0 ? (_jsx(Box, { ref: filterItemsContainerRef, sx: {
                                    display: "flex",
                                    gap: 1,
                                    overflowY: "hidden",
                                    overflowX: "auto",
                                    scrollbarWidth: "none",
                                }, children: filters.map((f, i) => (_jsx(Box, { sx: { flexShrink: 0, display: "flex" }, children: _jsx(Searchable, { index: i, definition: [
                                            ..._definitions,
                                            ...(searchMetadata ?? []),
                                        ]?.find((m) => m.name === f.name), onDelete: (i) => console.log(i), filters: filters, onFiltersChange: handleFiltersChange, isNew: f.isNew }) }, i))) })) : null, _jsxs(Box, { sx: {
                                    mr: "auto",
                                }, children: [_jsx(Tooltip, { title: t("nerdAdvancedSearch.add_filter"), size: "sm", children: _jsx(IconButton, { onClick: () => {
                                                setOpenFiltersList(true);
                                                setTimeout(() => inputRef.current?.focus());
                                            }, size: "sm", variant: "solid", color: "primary", ref: filtersButtonRef, children: _jsx("i", { className: "fa-sharp fa-solid fa-filter-list" }) }) }), _jsx(Popper, { open: openFiltersList, anchorEl: filtersButtonRef.current, placement: "bottom-start", style: {
                                            width: sm ? "calc(100% - 32px)" : undefined,
                                            zIndex: theme.zIndex.modal,
                                        }, children: _jsx(ClickAwayListener, { onClickAway: () => {
                                                setOpenFiltersList(false);
                                                setSearchValue("");
                                            }, children: _jsxs(Card, { size: "sm", children: [_jsx(Input, { autoFocus: true, size: "sm", slotProps: {
                                                            input: {
                                                                ref: inputRef,
                                                            },
                                                        }, onChange: (e) => setSearchValue(e.target.value), onKeyDown: (e) => {
                                                            e.stopPropagation();
                                                        } }), _jsx(List, { sx: {
                                                            maxHeight: 400,
                                                            overflow: "auto",
                                                        }, children: _definitions
                                                            ? (searchValue
                                                                ? _definitions.filter((d) => d.label
                                                                    .trim()
                                                                    .toLowerCase()
                                                                    .includes(searchValue.trim().toLowerCase()))
                                                                : _definitions).map((s, i) => (_jsxs(ListItemButton, { onClick: () => {
                                                                    handleAddField(s);
                                                                    setOpenFiltersList(false);
                                                                    setSearchValue("");
                                                                }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: s.faIcon ?? genericIcons[s.type] }) }), s.label] }, i)))
                                                            : null })] }) }) })] }), _jsxs(Box, { sx: { display: "flex", gap: 1 }, children: [_jsx(Tooltip, { title: t("nerdAdvancedSearch.refresh"), size: "sm", children: _jsx(IconButton, { onClick: () => {
                                                onRefresh({
                                                    filters: searchParams.get("filters"),
                                                    offset: searchParams.get("offset") ?? 0,
                                                    ...(searchParams.get("limit")
                                                        ? { limit: searchParams.get("limit") }
                                                        : itemsPerPage !== null
                                                            ? { limit: itemsPerPage || 50 }
                                                            : {}),
                                                });
                                            }, variant: "solid", color: "primary", size: "sm", disabled: refreshPending, children: refreshPending ? (_jsx(CircularProgress, { sx: {
                                                    "--CircularProgress-size": "18px",
                                                    "--CircularProgress-trackThickness": "3px",
                                                    "--CircularProgress-progressThickness": "3px",
                                                } })) : (_jsx("i", { className: "fa-regular fa-arrows-rotate" })) }) }), _jsxs(Dropdown, { children: [_jsx(MenuButton, { slots: { root: IconButton }, slotProps: {
                                                    root: {
                                                        size: "sm",
                                                        variant: "solid",
                                                        color: "primary",
                                                    },
                                                }, children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsxs(Menu, { sx: {
                                                    zIndex: (theme) => theme.zIndex.modal,
                                                }, children: [onNewSearchPreset ? (_jsxs(MenuItem, { onClick: () => {
                                                            setOpenPresetForm(DEFAULT_SEARCH_PRESET);
                                                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-floppy-disk" }) }), t("nerdAdvancedSearch.save_as_preset")] })) : ([]), _jsxs(MenuItem, { onClick: () => {
                                                            _setVariant((s) => s === "dialog" ? "toolbar" : "dialog");
                                                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-up-right-and-down-left-from-center" }) }), t("nerdAdvancedSearch.expand")] }), (searchMetadata?.length ?? 0) > 0 ? (_jsx(Tooltip, { title: t("nerdAdvancedSearch.uses_search_metadata"), size: "sm", children: _jsxs(MenuItem, { onClick: () => {
                                                                _setAdvancedMode((s) => !s);
                                                            }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-hammer" }) }), _advancedMode
                                                                    ? t("nerdAdvancedSearch.turn_dev_mode_off")
                                                                    : t("nerdAdvancedSearch.dev_mode")] }) })) : ([]), _jsxs(MenuItem, { onClick: () => {
                                                            clearAll();
                                                        }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-trash" }) }), t("nerdAdvancedSearch.clear_all")] })] })] })] })] })) : (_jsxs(Dialog, { open: _open, onClose: () => _setOpen(false), maxWidth: !sm ? "sm" : undefined, fullWidth: true, fullScreen: sm, children: [_jsxs(Sheet, { variant: "soft", sx: {
                                    p: 2,
                                    display: "flex",
                                    flexDirection: "column",
                                    gap: 1,
                                    flex: sm ? 1 : undefined,
                                }, children: [_jsxs(Box, { sx: { display: "flex", justifyContent: "space-between" }, children: [_jsx(Typography, { level: "title-md", sx: { color: "inherit" }, startDecorator: _jsx("i", { className: "fa-solid fa-magnifying-glass" }), children: t("nerdAdvancedSearch.search") }), _jsxs(Dropdown, { children: [_jsx(MenuButton, { slots: { root: IconButton }, slotProps: {
                                                            root: {
                                                                size: "sm",
                                                                variant: "solid",
                                                                color: "primary",
                                                            },
                                                        }, children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsxs(Menu, { sx: {
                                                            zIndex: (theme) => theme.zIndex.modal,
                                                        }, children: [onNewSearchPreset ? (_jsxs(MenuItem, { onClick: () => {
                                                                    setOpenPresetForm(DEFAULT_SEARCH_PRESET);
                                                                }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-floppy-disk" }) }), t("nerdAdvancedSearch.save_as_preset")] })) : ([]), _jsxs(MenuItem, { onClick: () => {
                                                                    _setVariant((s) => s === "dialog" ? "toolbar" : "dialog");
                                                                }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-down-left-and-up-right-to-center" }) }), t("nerdAdvancedSearch.reduce")] }), (searchMetadata?.length ?? 0) > 0 ? (_jsx(Tooltip, { title: t("nerdAdvancedSearch.uses_search_metadata"), size: "sm", children: _jsxs(MenuItem, { onClick: () => {
                                                                        _setAdvancedMode((s) => !s);
                                                                    }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-hammer" }) }), _advancedMode
                                                                            ? t("nerdAdvancedSearch.turn_dev_mode_off")
                                                                            : t("nerdAdvancedSearch.dev_mode")] }) })) : ([]), _jsxs(MenuItem, { onClick: () => {
                                                                    clearAll();
                                                                }, children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: "fa-solid fa-trash" }) }), t("nerdAdvancedSearch.clear_all")] })] })] })] }), _jsx(Typography, { level: "body-sm", sx: { color: "inherit" }, children: _jsx(Trans, { i18n: nerdUII18n, i18nKey: "components:nerdAdvancedSearch.description" }) }), _jsx(Box, { sx: {
                                            display: "flex",
                                            flexDirection: "column",
                                            gap: 1,
                                            alignItems: "stretch",
                                        }, children: filters.map((f, i) => (_jsx(Searchable, { index: i, definition: _definitions?.find((m) => m.name === f.name), onDelete: (i) => console.log(i), filters: filters, onFiltersChange: handleFiltersChange, isNew: f.isNew }, i))) }), _jsx(Box, { sx: { display: "flex", justifyContent: "center" }, children: _jsxs(Dropdown, { children: [_jsx(Tooltip, { title: t("nerdAdvancedSearch.add_filter"), size: "sm", children: _jsx(MenuButton, { slots: { root: IconButton }, slotProps: {
                                                            root: {
                                                                size: "sm",
                                                                variant: "solid",
                                                                color: "primary",
                                                            },
                                                        }, children: _jsx("i", { className: "fa-sharp fa-solid fa-filter-list" }) }) }), _jsx(Menu, { placement: "bottom-start", sx: {
                                                        maxHeight: 400,
                                                        overflow: "auto",
                                                        width: sm ? "calc(100% - 32px)" : undefined,
                                                        zIndex: (theme) => theme.zIndex.modal,
                                                    }, children: _definitions?.map((s, i) => (_jsxs(MenuItem, { onClick: () => handleAddField(s), children: [_jsx(ListItemDecorator, { sx: { color: "inherit" }, children: _jsx("i", { className: s.faIcon ?? genericIcons[s.type] }) }), s.label] }, i))) })] }) })] }), _jsxs(DialogActions, { children: [_jsx(Button, { size: "sm", onClick: () => _setOpen(false), variant: "soft", color: "neutral", children: t("nerdAdvancedSearch.close") }), _jsx(Tooltip, { title: t("nerdAdvancedSearch.enter"), size: "sm", children: _jsx(Button, { endDecorator: _jsx("i", { className: "fa-sharp fa-solid fa-arrow-right" }), size: "sm", color: "primary", children: t("nerdAdvancedSearch.go") }) })] })] }))] }), onNewSearchPreset && onDeleteSearchPreset ? (_jsx(SearchPresetForm, { searchPreset: openPresetForm, onClose: () => setOpenPresetForm(null), filters: filters, definitions: _definitions, onNewSearchPreset: onNewSearchPreset, onDelete: onDeleteSearchPreset })) : null] }));
}
const Presets = ({ searchPresets, currentUserUUID, setFilters, setOpenPresetForm, }) => {
    const { t } = useTranslation("components", { i18n: nerdUII18n });
    const dispatch = useDispatch();
    const buttonRef = useRef(null);
    const [openPresets, setOpenPresets] = useState(false);
    const selectedPresets = useSelector(nerdAdvancedSearchSelectedPresetsSelector);
    const gSearchPresets = useMemo(() => searchPresets?.filter((s) => s.global), [searchPresets]);
    const mySearchPresets = useMemo(() => searchPresets?.filter((s) => {
        return !s.global && new SearchPreset(s).isMine(currentUserUUID || "");
    }), [searchPresets, currentUserUUID]);
    const handleClose = () => {
        setOpenPresets(false);
    };
    const handleListKeyDown = (event) => {
        if (event.key === "Tab") {
            setOpenPresets(false);
        }
        else if (event.key === "Escape") {
            buttonRef.current?.focus();
            setOpenPresets(false);
        }
    };
    const handleUpdatePresets = (selectedPresets) => {
        const filters = [];
        try {
            selectedPresets.forEach((id) => {
                const preset = searchPresets?.find((p) => p.id === id);
                if (preset)
                    filters.push(...JSON.parse(preset.filter));
            });
            setFilters(filters);
            dispatch(setSelectedPresets(selectedPresets));
        }
        catch {
            setFilters([]);
            dispatch(setSelectedPresets(selectedPresets));
        }
    };
    const emptyPresets = (_jsxs(Grid, { container: true, spacing: 1, textAlign: "center", children: [_jsx(Grid, { xs: 12, children: _jsx(Typography, { level: "h2", color: "neutral", children: _jsx("i", { className: "fa-solid fa-inbox" }) }) }), _jsxs(Grid, { xs: 12, children: [_jsx(Typography, { children: "No Presets." }), _jsx(Typography, { level: "body-sm", display: "inline", children: "Add filters" }), _jsxs(Typography, { level: "body-sm", display: "inline", color: "primary", children: [" ", _jsx("i", { className: "fa-sharp fa-solid fa-filter-list" }), " "] }), _jsx(Typography, { level: "body-sm", display: "inline", children: "and click on" }), _jsx(Typography, { level: "body-sm", color: "primary", display: "inline", children: _jsxs("b", { children: [" ", _jsx("i", { className: "fa-solid fa-floppy-disk" }), " Save as preset"] }) }), _jsxs(Typography, { level: "body-sm", display: "inline", children: [" ", "in the menu", " "] }), _jsxs(Typography, { level: "body-sm", color: "primary", display: "inline", children: [" ", _jsx("i", { className: "fa-solid fa-ellipsis-vertical" })] })] })] }));
    const Item = (s) => {
        const selected = selectedPresets.includes(s.id);
        return (_jsxs(Fragment, { children: [_jsx(ListItem, { endAction: _jsxs(_Fragment, { children: [currentUserUUID === s.userUUID ? (_jsx(Tooltip, { title: "Edit", size: "sm", children: _jsx(IconButton, { "aria-label": "Edit", size: "sm", onClick: () => {
                                        setOpenPresetForm(s);
                                        setOpenPresets(false);
                                    }, children: _jsx("i", { className: "fa-solid fa-pen-to-square" }) }) })) : null, selected ? (_jsx(Tooltip, { title: "Unselect", size: "sm", children: _jsx(IconButton, { color: "danger", size: "sm", onClick: () => {
                                        const _selectedPresets = [...selectedPresets];
                                        const index = selectedPresets.findIndex((id) => id === s.id);
                                        _selectedPresets.splice(index, 1);
                                        handleUpdatePresets(_selectedPresets);
                                    }, children: _jsx("i", { className: "fa-solid fa-xmark" }) }) })) : null] }), children: _jsxs(ListItemButton, { sx: {
                            p: 1,
                        }, onClick: () => {
                            if (!selected) {
                                handleUpdatePresets([...selectedPresets, s.id]);
                            }
                        }, children: [_jsx(ListItemDecorator, { children: _jsx(Box, { sx: {
                                        width: 20,
                                        height: 20,
                                        borderRadius: "sm",
                                        bgcolor: s.color,
                                    } }) }), _jsx(ListItemContent, { children: s.name })] }) }), _jsx(ListDivider, { inset: "startContent" })] }, s.id));
    };
    return (_jsxs("div", { children: [_jsx(Tooltip, { title: t("components:nerdAdvancedSearch.presets"), size: "sm", children: _jsx(Badge, { size: "md", badgeContent: selectedPresets.length, children: _jsx(IconButton, { ref: buttonRef, size: "sm", variant: "solid", color: "primary", onClick: () => {
                            setOpenPresets(!openPresets);
                        }, children: _jsx("i", { className: "fa-solid fa-rectangle-vertical-history" }) }) }) }), _jsx(Popper, { role: undefined, open: openPresets, anchorEl: buttonRef.current, disablePortal: true, modifiers: [
                    {
                        name: "offset",
                        options: {
                            offset: [0, 4],
                        },
                    },
                ], sx: {
                    zIndex: (theme) => theme.zIndex.modal,
                }, children: _jsx(ClickAwayListener, { onClickAway: handleClose, children: _jsx(MenuList, { component: "div", variant: "outlined", size: "sm", sx: {
                            boxShadow: "sm",
                            borderRadius: "sm",
                            flexGrow: 0,
                            minWidth: 200,
                            maxWidth: 300,
                            maxHeight: 300,
                            overflow: "auto",
                            p: 1,
                        }, onKeyDown: handleListKeyDown, children: _jsxs(Tabs, { defaultValue: 0, sx: { bgcolor: "transparent", overflowY: "auto" }, children: [_jsxs(TabList, { sticky: "top", disableUnderline: true, sx: {
                                        p: 0.5,
                                        gap: 0.5,
                                        borderRadius: "xl",
                                        bgcolor: "background.level1",
                                        [`& .${tabClasses.root}[aria-selected="true"]`]: {
                                            boxShadow: "sm",
                                            bgcolor: "background.surface",
                                        },
                                    }, children: [_jsx(Tab, { disableIndicator: true, sx: {
                                                flex: 1,
                                            }, children: "Global Presets" }), _jsx(Tab, { disableIndicator: true, children: "My Presets" })] }), _jsx(TabPanel, { value: 0, children: gSearchPresets && gSearchPresets.length > 0 ? (_jsx(List, { children: gSearchPresets.map((f) => (_jsx(Item, { ...f }, f.id))) })) : (emptyPresets) }), _jsx(TabPanel, { value: 1, children: mySearchPresets && mySearchPresets.length > 0 ? (_jsx(List, { children: mySearchPresets.map((f) => (_jsx(Item, { ...f }, f.id))) })) : (emptyPresets) })] }) }) }) })] }));
};
