import Cookies from "js-cookie";
import { useEffect, useState } from "react";
/**
 *
 * @param {string} key cookie key
 * @param {string} initialValue initial value
 * @param {Cookies.CookieAttributes} options options
 * @param {number} interval refresh interval
 * @returns {[string | undefined,(value: string, options?: Cookies.CookieAttributes) => void]} useCookie
 */
export function useCookie(key, initialValue, options = { path: "/" }, interval = 1000) {
    const [cookie, setCookie] = useState(Cookies.get(key) || initialValue);
    useEffect(() => {
        const checkCookieChange = () => {
            const currentCookie = Cookies.get(key);
            if (currentCookie !== cookie) {
                setCookie(currentCookie || initialValue);
            }
        };
        const intervalId = setInterval(checkCookieChange, interval);
        return () => {
            clearInterval(intervalId);
        };
    }, [interval]);
    const updateCookie = (value) => {
        if (value)
            Cookies.set(key, value, options);
        else
            Cookies.remove(key);
        setCookie(value);
    };
    return [cookie, updateCookie];
}
